@@ -9,10 +9,14 @@ use crate::packet::{
 };
 use crate::Result;
 use crate::packet::Signature;
-use crate::packet::signature;
+use crate::packet::signature::{
+    self,
+    SignatureBuilder,
+    subpacket::SubpacketTag,
+};
 use crate::cert::prelude::*;
 use crate::Error;
-use crate::crypto::Password;
+use crate::crypto::{Password, Signer};
 use crate::types::{
     Features,
     HashAlgorithm,
@@ -168,9 +172,9 @@ pub struct CertBuilder<'a> {
     creation_time: Option<std::time::SystemTime>,
     ciphersuite: CipherSuite,
     primary: KeyBlueprint,
-    subkeys: Vec<KeyBlueprint>,
-    userids: Vec<packet::UserID>,
-    user_attributes: Vec<packet::UserAttribute>,
+    subkeys: Vec<(Option<SignatureBuilder>, KeyBlueprint)>,
+    userids: Vec<(Option<SignatureBuilder>, packet::UserID)>,
+    user_attributes: Vec<(Option<SignatureBuilder>, packet::UserAttribute)>,
     password: Option<Password>,
     revocation_keys: Option<Vec<RevocationKey>>,
     phantom: PhantomData<&'a ()>,
@@ -254,40 +258,24 @@ impl CertBuilder<'_> {
     /// # Ok(())
     /// # }
     /// ```
-    pub fn general_purpose<C, U>(ciphersuite: C, userids: Option<U>) -> Self
+    pub fn general_purpose<C, U>(ciphersuite: C, userid: Option<U>) -> Self
         where C: Into<Option<CipherSuite>>,
               U: Into<packet::UserID>
     {
-        CertBuilder {
-            creation_time: None,
-            ciphersuite: ciphersuite.into().unwrap_or(Default::default()),
-            primary: KeyBlueprint {
-                flags: KeyFlags::empty()
-                    .set_certification(),
-                validity: Some(time::Duration::new(3 * 52 * 7 * 24 * 60 * 60, 0)),
-                ciphersuite: None,
-            },
-            subkeys: vec![
-                KeyBlueprint {
-                    flags: KeyFlags::empty()
-                        .set_signing(),
-                    validity: None,
-                    ciphersuite: None,
-                },
-                KeyBlueprint {
-                    flags: KeyFlags::empty()
+        let mut builder = Self::new()
+            .set_cipher_suite(ciphersuite.into().unwrap_or_default())
+            .set_primary_key_flags(KeyFlags::empty().set_certification())
+            .set_validity_period(
+                time::Duration::new(3 * 52 * 7 * 24 * 60 * 60, 0))
+            .add_signing_subkey()
+            .add_subkey(KeyFlags::empty()
                         .set_transport_encryption()
-                        .set_storage_encryption(),
-                    validity: None,
-                    ciphersuite: None,
-                }
-            ],
-            userids: userids.into_iter().map(|x| x.into()).collect(),
-            user_attributes: vec![],
-            password: None,
-            revocation_keys: None,
-            phantom: PhantomData,
+                        .set_storage_encryption(), None, None);
+        if let Some(u) = userid.map(Into::into) {
+            builder = builder.add_userid(u);
         }
+
+        builder
     }
 
     /// Sets the creation time.
@@ -460,10 +448,107 @@ impl CertBuilder<'_> {
     pub fn add_userid<U>(mut self, uid: U) -> Self
         where U: Into<packet::UserID>
     {
-        self.userids.push(uid.into());
+        self.userids.push((None, uid.into()));
         self
     }
 
+    /// Adds a User ID with a binding signature based on `builder`.
+    ///
+    /// Adds a User ID to the certificate, creating the binding
+    /// signature using `builder`.  The `builder`s signature type must
+    /// be a certification signature (i.e. either
+    /// [`GenericCertification`], [`PersonaCertification`],
+    /// [`CasualCertification`], or [`PositiveCertification`]).
+    ///
+    /// The key generation step uses `builder` as a template, but
+    /// tweaks it so the signature is a valid binding signature.  If
+    /// you need more control, consider using
+    /// [`UserID::bind`](crate::packet::UserID::bind).
+    ///
+    /// The following modifications are performed on `builder`:
+    ///
+    ///   - An appropriate hash algorithm is selected.
+    ///
+    ///   - The creation time is set.
+    ///
+    ///   - Primary key metadata is added (key flags, key validity period).
+    ///
+    ///   - Certificate metadata is added (feature flags, algorithm
+    ///     preferences).
+    ///
+    ///   - The [`CertBuilder`] marks exactly one User ID or User
+    ///     Attribute as primary: The first one provided to
+    ///     [`CertBuilder::add_userid_with`] or
+    ///     [`CertBuilder::add_user_attribute_with`] (the UserID takes
+    ///     precedence) that is marked as primary, or the first User
+    ///     ID or User Attribute added to the [`CertBuilder`].
+    ///
+    ///   [`GenericCertification`]: crate::types::SignatureType::GenericCertification
+    ///   [`PersonaCertification`]: crate::types::SignatureType::PersonaCertification
+    ///   [`CasualCertification`]: crate::types::SignatureType::CasualCertification
+    ///   [`PositiveCertification`]: crate::types::SignatureType::PositiveCertification
+    ///   [primary User ID flag]: https://tools.ietf.org/html/rfc4880#section-5.2.3.19
+    ///
+    /// # Examples
+    ///
+    /// This example very casually binds a User ID to a certificate.
+    ///
+    /// ```
+    /// # fn main() -> sequoia_openpgp::Result<()> {
+    /// # use sequoia_openpgp as openpgp;
+    /// # use openpgp::cert::prelude::*;
+    /// # use openpgp::packet::{prelude::*, signature::subpacket::*};
+    /// # use openpgp::policy::StandardPolicy;
+    /// # use openpgp::types::*;
+    /// # let policy = &StandardPolicy::new();
+    /// #
+    /// let (cert, revocation_cert) =
+    ///     CertBuilder::general_purpose(
+    ///         None, Some("Alice Lovelace <alice@example.org>"))
+    ///     .add_userid_with(
+    ///         "trinity",
+    ///         SignatureBuilder::new(SignatureType::CasualCertification)
+    ///             .set_notation("rabbit@example.org", b"follow me",
+    ///                           NotationDataFlags::empty().set_human_readable(),
+    ///                           false)?)?
+    ///     .generate()?;
+    ///
+    /// assert_eq!(cert.userids().count(), 2);
+    /// let mut userids = cert.with_policy(policy, None)?.userids().collect::<Vec<_>>();
+    /// // Sort lexicographically.
+    /// userids.sort_by(|a, b| a.value().cmp(b.value()));
+    /// assert_eq!(userids[0].userid(),
+    ///            &UserID::from("Alice Lovelace <alice@example.org>"));
+    /// assert_eq!(userids[1].userid(),
+    ///            &UserID::from("trinity"));
+    ///
+    /// assert!(userids[0].binding_signature().primary_userid().unwrap_or(false));
+    /// assert!(! userids[1].binding_signature().primary_userid().unwrap_or(false));
+    /// assert_eq!(userids[1].binding_signature().notation("rabbit@example.org")
+    ///            .next().unwrap(), b"follow me");
+    /// # Ok(()) }
+    /// ```
+    pub fn add_userid_with<U, B>(mut self, uid: U, builder: B)
+                                 -> Result<Self>
+    where U: Into<packet::UserID>,
+          B: Into<SignatureBuilder>,
+    {
+        let builder = builder.into();
+        match builder.typ() {
+            SignatureType::GenericCertification
+                | SignatureType::PersonaCertification
+                | SignatureType::CasualCertification
+                | SignatureType::PositiveCertification =>
+            {
+                self.userids.push((Some(builder), uid.into()));
+                Ok(self)
+            },
+            t =>
+                Err(Error::InvalidArgument(format!(
+                    "Signature type is not a certification: {}", t)).into()),
+        }
+    }
+
     /// Adds a new User Attribute.
     ///
     /// Adds a User Attribute to the certificate.  If there are no
@@ -540,10 +625,106 @@ impl CertBuilder<'_> {
     pub fn add_user_attribute<U>(mut self, ua: U) -> Self
         where U: Into<packet::UserAttribute>
     {
-        self.user_attributes.push(ua.into());
+        self.user_attributes.push((None, ua.into()));
         self
     }
 
+    /// Adds a User Attribute with a binding signature based on `builder`.
+    ///
+    /// Adds a User Attribute to the certificate, creating the binding
+    /// signature using `builder`.  The `builder`s signature type must
+    /// be a certification signature (i.e. either
+    /// [`GenericCertification`], [`PersonaCertification`],
+    /// [`CasualCertification`], or [`PositiveCertification`]).
+    ///
+    /// The key generation step uses `builder` as a template, but
+    /// tweaks it so the signature is a valid binding signature.  If
+    /// you need more control, consider using
+    /// [`UserAttribute::bind`](crate::packet::UserAttribute::bind).
+    ///
+    /// The following modifications are performed on `builder`:
+    ///
+    ///   - An appropriate hash algorithm is selected.
+    ///
+    ///   - The creation time is set.
+    ///
+    ///   - Primary key metadata is added (key flags, key validity period).
+    ///
+    ///   - Certificate metadata is added (feature flags, algorithm
+    ///     preferences).
+    ///
+    ///   - The [`CertBuilder`] marks exactly one User ID or User
+    ///     Attribute as primary: The first one provided to
+    ///     [`CertBuilder::add_userid_with`] or
+    ///     [`CertBuilder::add_user_attribute_with`] (the UserID takes
+    ///     precedence) that is marked as primary, or the first User
+    ///     ID or User Attribute added to the [`CertBuilder`].
+    ///
+    ///   [`GenericCertification`]: crate::types::SignatureType::GenericCertification
+    ///   [`PersonaCertification`]: crate::types::SignatureType::PersonaCertification
+    ///   [`CasualCertification`]: crate::types::SignatureType::CasualCertification
+    ///   [`PositiveCertification`]: crate::types::SignatureType::PositiveCertification
+    ///   [primary User ID flag]: https://tools.ietf.org/html/rfc4880#section-5.2.3.19
+    ///
+    /// # Examples
+    ///
+    /// This example very casually binds a user attribute to a
+    /// certificate.
+    ///
+    /// ```
+    /// # fn main() -> sequoia_openpgp::Result<()> {
+    /// # use sequoia_openpgp as openpgp;
+    /// # use openpgp::packet::user_attribute::Subpacket;
+    /// # use openpgp::cert::prelude::*;
+    /// # use openpgp::packet::{prelude::*, signature::subpacket::*};
+    /// # use openpgp::policy::StandardPolicy;
+    /// # use openpgp::types::*;
+    /// #
+    /// # let policy = &StandardPolicy::new();
+    /// #
+    /// # // Create some user attribute. Doctests do not pass cfg(test),
+    /// # // so UserAttribute::arbitrary is not available
+    /// # let user_attribute =
+    /// #   UserAttribute::new(&[Subpacket::Unknown(7, vec![7; 7].into())])?;
+    /// let (cert, revocation_cert) =
+    ///     CertBuilder::general_purpose(
+    ///         None, Some("Alice Lovelace <alice@example.org>"))
+    ///     .add_user_attribute_with(
+    ///         user_attribute,
+    ///         SignatureBuilder::new(SignatureType::CasualCertification)
+    ///             .set_notation("rabbit@example.org", b"follow me",
+    ///                           NotationDataFlags::empty().set_human_readable(),
+    ///                           false)?)?
+    ///     .generate()?;
+    ///
+    /// let uas = cert.with_policy(policy, None)?.user_attributes().collect::<Vec<_>>();
+    /// assert_eq!(uas.len(), 1);
+    /// assert!(! uas[0].binding_signature().primary_userid().unwrap_or(false));
+    /// assert_eq!(uas[0].binding_signature().notation("rabbit@example.org")
+    ///            .next().unwrap(), b"follow me");
+    /// # Ok(()) }
+    /// ```
+    pub fn add_user_attribute_with<U, B>(mut self, ua: U, builder: B)
+                                         -> Result<Self>
+    where U: Into<packet::UserAttribute>,
+          B: Into<SignatureBuilder>,
+    {
+        let builder = builder.into();
+        match builder.typ() {
+            SignatureType::GenericCertification
+                | SignatureType::PersonaCertification
+                | SignatureType::CasualCertification
+                | SignatureType::PositiveCertification =>
+            {
+                self.user_attributes.push((Some(builder), ua.into()));
+                Ok(self)
+            },
+            t =>
+                Err(Error::InvalidArgument(format!(
+                    "Signature type is not a certification: {}", t)).into()),
+        }
+    }
+
     /// Adds a signing-capable subkey.
     ///
     /// The key uses the default cipher suite (see
@@ -790,14 +971,94 @@ impl CertBuilder<'_> {
         where T: Into<Option<time::Duration>>,
               C: Into<Option<CipherSuite>>,
     {
-        self.subkeys.push(KeyBlueprint {
+        self.subkeys.push((None, KeyBlueprint {
             flags,
             validity: validity.into(),
             ciphersuite: cs.into(),
-        });
+        }));
         self
     }
 
+    /// Adds a subkey with a binding signature based on `builder`.
+    ///
+    /// Adds a subkey to the certificate, creating the binding
+    /// signature using `builder`.  The `builder`s signature type must
+    /// be [`SubkeyBinding`].
+    ///
+    /// The key generation step uses `builder` as a template, but adds
+    /// all subpackets that the signature needs to be a valid binding
+    /// signature.  If you need more control, or want to adopt
+    /// existing keys, consider using
+    /// [`Key::bind`](crate::packet::Key::bind).
+    ///
+    /// The following modifications are performed on `builder`:
+    ///
+    ///   - An appropriate hash algorithm is selected.
+    ///
+    ///   - The creation time is set.
+    ///
+    ///   - Key metadata is added (key flags, key validity period).
+    ///
+    ///   [`SubkeyBinding`]: crate::types::SignatureType::SubkeyBinding
+    ///
+    /// # Examples
+    ///
+    /// This example binds a signing subkey to a certificate,
+    /// restricting its use to authentication of software.
+    ///
+    /// ```
+    /// # fn main() -> sequoia_openpgp::Result<()> {
+    /// # use sequoia_openpgp as openpgp;
+    /// # use openpgp::packet::user_attribute::Subpacket;
+    /// # use openpgp::cert::prelude::*;
+    /// # use openpgp::packet::{prelude::*, signature::subpacket::*};
+    /// # use openpgp::policy::StandardPolicy;
+    /// # use openpgp::types::*;
+    /// let (cert, revocation_cert) =
+    ///     CertBuilder::general_purpose(
+    ///         None, Some("Alice Lovelace <alice@example.org>"))
+    ///     .add_subkey_with(
+    ///         KeyFlags::empty().set_signing(), None, None,
+    ///         SignatureBuilder::new(SignatureType::SubkeyBinding)
+    ///             // Add a critical notation!
+    ///             .set_notation("code-signing@policy.example.org", b"",
+    ///                           NotationDataFlags::empty(), true)?)?
+    ///     .generate()?;
+    ///
+    /// // Under the standard policy, the additional signing subkey
+    /// // is not bound.
+    /// let p = StandardPolicy::new();
+    /// assert_eq!(cert.with_policy(&p, None)?.keys().for_signing().count(), 1);
+    ///
+    /// // However, software implementing the notation see the additional
+    /// // signing subkey.
+    /// let mut p = StandardPolicy::new();
+    /// p.good_critical_notations(&["code-signing@policy.example.org"]);
+    /// assert_eq!(cert.with_policy(&p, None)?.keys().for_signing().count(), 2);
+    /// # Ok(()) }
+    /// ```
+    pub fn add_subkey_with<T, C, B>(mut self, flags: KeyFlags, validity: T,
+                                    cs: C, builder: B) -> Result<Self>
+        where T: Into<Option<time::Duration>>,
+              C: Into<Option<CipherSuite>>,
+              B: Into<SignatureBuilder>,
+    {
+        let builder = builder.into();
+        match builder.typ() {
+            SignatureType::SubkeyBinding => {
+                self.subkeys.push((Some(builder), KeyBlueprint {
+                    flags,
+                    validity: validity.into(),
+                    ciphersuite: cs.into(),
+                }));
+                Ok(self)
+            },
+            t =>
+                Err(Error::InvalidArgument(format!(
+                    "Signature type is not a subkey binding: {}", t)).into()),
+        }
+    }
+
     /// Sets the primary key's key flags.
     ///
     /// By default, the primary key is set to only be certification
@@ -975,81 +1236,114 @@ impl CertBuilder<'_> {
                     time::Duration::new(SIG_BACKDATE_BY, 0)
             });
 
-        let mut packets = Vec::<Packet>::with_capacity(
-            1 + 1 + self.subkeys.len() + self.userids.len()
-                + self.user_attributes.len());
-
         // make sure the primary key can sign subkeys
         if !self.subkeys.is_empty() {
             self.primary.flags = self.primary.flags.set_certification();
         }
 
         // Generate & self-sign primary key.
-        let (primary, sig) = self.primary_key(creation_time)?;
-        let mut signer = primary.clone().into_keypair().unwrap();
+        let (primary, sig, mut signer) = self.primary_key(creation_time)?;
 
-        packets.push(Packet::SecretKey({
-            let mut primary = primary.clone();
-            if let Some(ref password) = self.password {
-                primary.secret_mut().encrypt_in_place(password)?;
-            }
-            primary
-        }));
-        packets.push(sig.clone().into());
+        let mut cert = Cert::try_from(vec![
+            Packet::SecretKey({
+                let mut primary = primary.clone();
+                if let Some(ref password) = self.password {
+                    primary.secret_mut().encrypt_in_place(password)?;
+                }
+                primary
+            }),
+            sig.into(),
+        ])?;
 
-        let sig = signature::SignatureBuilder::from(sig.clone())
-            .set_signature_creation_time(creation_time)?;
+        // We want to mark exactly one User ID or Attribute as primary.
+        // First, figure out whether one of the binding signature
+        // templates have the primary flag set.
+        let have_primary_user_thing = {
+            let is_primary = |osig: &Option<SignatureBuilder>| -> bool {
+                osig.as_ref().and_then(|s| s.primary_userid()).unwrap_or(false)
+            };
 
-        // Remove subpackets that needn't be copied into the binding
-        // signatures.
-        let sig = sig.set_revocation_key(vec![])?;
-
-        let mut cert = Cert::try_from(packets)?;
-
-        let have_userids = self.userids.len() > 0;
+            self.userids.iter().map(|(s, _)| s).any(is_primary)
+                || self.user_attributes.iter().map(|(s, _)| s).any(is_primary)
+        };
+        let mut emitted_primary_user_thing = false;
 
         // Sign UserIDs.
-        for (i, uid) in self.userids.into_iter().enumerate() {
-            let mut builder = sig.clone()
-                .set_type(SignatureType::PositiveCertification)
-                // GnuPG wants at least a 512-bit hash for P521 keys.
-                .set_hash_algo(HashAlgorithm::SHA512);
-            if i == 0 {
-                builder = builder.set_primary_userid(true)?;
+        for (template, uid) in self.userids.into_iter() {
+            let sig = template.unwrap_or_else(
+                || SignatureBuilder::new(SignatureType::PositiveCertification));
+            let sig = Self::signature_common(sig, creation_time)?;
+            let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
+
+            // Make sure we mark exactly one User ID or Attribute as
+            // primary.
+            if emitted_primary_user_thing {
+                sig = sig.modify_hashed_area(|mut a| {
+                    a.remove_all(SubpacketTag::PrimaryUserID);
+                    Ok(a)
+                })?;
+            } else {
+                if have_primary_user_thing {
+                    // Check if this is the first explicitly selected
+                    // user thing.
+                    emitted_primary_user_thing |=
+                        sig.primary_userid().unwrap_or(false);
+                } else {
+                    // Implicitly mark the first as primary.
+                    sig = sig.set_primary_userid(true)?;
+                    emitted_primary_user_thing = true;
+                }
             }
-            let signature = uid.bind(&mut signer, &cert, builder)?;
+
+            let signature = uid.bind(&mut signer, &cert, sig)?;
             cert = cert.insert_packets(
                 vec![Packet::from(uid), signature.into()])?;
         }
 
         // Sign UserAttributes.
-        for (i, ua) in self.user_attributes.into_iter().enumerate() {
-            let mut builder = sig.clone()
-                .set_type(SignatureType::PositiveCertification)
-                 // GnuPG wants at least a 512-bit hash for P521 keys.
-                .set_hash_algo(HashAlgorithm::SHA512);
-            if i == 0 && ! have_userids {
-                builder = builder.set_primary_userid(true)?;
+        for (template, ua) in self.user_attributes.into_iter() {
+            let sig = template.unwrap_or_else(
+                || SignatureBuilder::new(SignatureType::PositiveCertification));
+            let sig = Self::signature_common(sig, creation_time)?;
+            let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
+
+            // Make sure we mark exactly one User ID or Attribute as
+            // primary.
+            if emitted_primary_user_thing {
+                sig = sig.modify_hashed_area(|mut a| {
+                    a.remove_all(SubpacketTag::PrimaryUserID);
+                    Ok(a)
+                })?;
+            } else {
+                if have_primary_user_thing {
+                    // Check if this is the first explicitly selected
+                    // user thing.
+                    emitted_primary_user_thing |=
+                        sig.primary_userid().unwrap_or(false);
+                } else {
+                    // Implicitly mark the first as primary.
+                    sig = sig.set_primary_userid(true)?;
+                    emitted_primary_user_thing = true;
+                }
             }
-            let signature = ua.bind(&mut signer, &cert, builder)?;
+
+            let signature = ua.bind(&mut signer, &cert, sig)?;
             cert = cert.insert_packets(
                 vec![Packet::from(ua), signature.into()])?;
         }
 
         // Sign subkeys.
-        for blueprint in self.subkeys {
+        for (template, blueprint) in self.subkeys {
             let flags = &blueprint.flags;
             let mut subkey = blueprint.ciphersuite
                 .unwrap_or(self.ciphersuite)
                 .generate_key(flags)?;
             subkey.set_creation_time(creation_time)?;
 
-            let mut builder =
-                signature::SignatureBuilder::new(SignatureType::SubkeyBinding)
-                .set_signature_creation_time(creation_time)?
-                // GnuPG wants at least a 512-bit hash for P521 keys.
-                .set_hash_algo(HashAlgorithm::SHA512)
-                .set_features(Features::sequoia())?
+            let sig = template.unwrap_or_else(
+                || SignatureBuilder::new(SignatureType::SubkeyBinding));
+            let sig = Self::signature_common(sig, creation_time)?;
+            let mut builder = sig
                 .set_key_flags(flags.clone())?
                 .set_key_validity_period(blueprint.validity.or(self.primary.validity))?;
 
@@ -1088,28 +1382,17 @@ impl CertBuilder<'_> {
         Ok((cert, revocation))
     }
 
+    /// Creates the primary key and a direct key signature.
     fn primary_key(&self, creation_time: std::time::SystemTime)
-        -> Result<(key::SecretKey, Signature)>
+        -> Result<(key::SecretKey, Signature, Box<dyn Signer>)>
     {
         let mut key = self.primary.ciphersuite
             .unwrap_or(self.ciphersuite)
             .generate_key(KeyFlags::empty().set_certification())?;
         key.set_creation_time(creation_time)?;
-        let mut sig = signature::SignatureBuilder::new(SignatureType::DirectKey)
-            // GnuPG wants at least a 512-bit hash for P521 keys.
-            .set_hash_algo(HashAlgorithm::SHA512)
-            .set_features(Features::sequoia())?
-            .set_key_flags(self.primary.flags.clone())?
-            .set_signature_creation_time(creation_time)?
-            .set_key_validity_period(self.primary.validity)?
-            .set_preferred_hash_algorithms(vec![
-                HashAlgorithm::SHA512,
-                HashAlgorithm::SHA256,
-            ])?
-            .set_preferred_symmetric_algorithms(vec![
-                SymmetricAlgorithm::AES256,
-                SymmetricAlgorithm::AES128,
-            ])?;
+        let sig = SignatureBuilder::new(SignatureType::DirectKey);
+        let sig = Self::signature_common(sig, creation_time)?;
+        let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
 
         if let Some(ref revocation_keys) = self.revocation_keys {
             sig = sig.set_revocation_key(revocation_keys.clone())?;
@@ -1119,7 +1402,38 @@ impl CertBuilder<'_> {
             .expect("key generated above has a secret");
         let sig = sig.sign_direct_key(&mut signer, key.parts_as_public())?;
 
-        Ok((key, sig.into()))
+        Ok((key, sig, Box::new(signer)))
+    }
+
+    /// Common settings for generated signatures.
+    fn signature_common(builder: SignatureBuilder,
+                        creation_time: time::SystemTime)
+                        -> Result<SignatureBuilder>
+    {
+        builder
+            // GnuPG wants at least a 512-bit hash for P521 keys.
+            .set_hash_algo(HashAlgorithm::SHA512)
+            .set_signature_creation_time(creation_time)
+    }
+
+
+    /// Adds primary key metadata to the signature.
+    fn add_primary_key_metadata(builder: SignatureBuilder,
+                                primary: &KeyBlueprint)
+                                -> Result<SignatureBuilder>
+    {
+        builder
+            .set_features(Features::sequoia())?
+            .set_key_flags(primary.flags.clone())?
+            .set_key_validity_period(primary.validity)?
+            .set_preferred_hash_algorithms(vec![
+                HashAlgorithm::SHA512,
+                HashAlgorithm::SHA256,
+            ])?
+            .set_preferred_symmetric_algorithms(vec![
+                SymmetricAlgorithm::AES256,
+                SymmetricAlgorithm::AES128,
+            ])
     }
 }
 
@@ -1311,15 +1625,13 @@ mod tests {
         assert!(! sig.key_alive(key, now + 610 * s).is_ok());
 
         let ka = cert.keys().with_policy(p, now).alive().revoked(false)
-            .for_signing()
-            .nth(0).unwrap();
+            .for_signing().next().unwrap();
         assert!(ka.alive().is_ok());
         assert!(ka.clone().with_policy(p, now + 290 * s).unwrap().alive().is_ok());
         assert!(! ka.clone().with_policy(p, now + 310 * s).unwrap().alive().is_ok());
 
         let ka = cert.keys().with_policy(p, now).alive().revoked(false)
-            .for_authentication()
-            .nth(0).unwrap();
+            .for_authentication().next().unwrap();
         assert!(ka.alive().is_ok());
         assert!(ka.clone().with_policy(p, now + 590 * s).unwrap().alive().is_ok());
         assert!(! ka.clone().with_policy(p, now + 610 * s).unwrap().alive().is_ok());
@@ -1424,4 +1736,91 @@ mod tests {
                    revokers.iter().collect::<HashSet<_>>());
         Ok(())
     }
+
+    /// Checks that the builder emits exactly one user id or attribute
+    /// marked as primary.
+    #[test]
+    fn primary_user_things() -> Result<()> {
+        fn count_primary_user_things(c: Cert) -> usize {
+            c.into_packets().map(|p| match p {
+                Packet::Signature(s) if s.primary_userid().unwrap_or(false)
+                    => 1,
+                _ => 0,
+            }).sum()
+        }
+
+        use crate::packet::{prelude::*, user_attribute::Subpacket};
+        let ua_foo =
+            UserAttribute::new(&[Subpacket::Unknown(7, vec![7; 7].into())])?;
+        let ua_bar =
+            UserAttribute::new(&[Subpacket::Unknown(11, vec![11; 11].into())])?;
+
+        let p = &P::new();
+        let positive = SignatureType::PositiveCertification;
+
+        let (c, _) = CertBuilder::new().generate()?;
+        assert_eq!(count_primary_user_things(c), 0);
+
+        let (c, _) = CertBuilder::new()
+            .add_userid("foo")
+            .generate()?;
+        assert_eq!(count_primary_user_things(c), 1);
+
+        let (c, _) = CertBuilder::new()
+            .add_userid("foo")
+            .add_userid("bar")
+            .generate()?;
+        assert_eq!(count_primary_user_things(c), 1);
+
+        let (c, _) = CertBuilder::new()
+            .add_user_attribute(ua_foo.clone())
+            .generate()?;
+        assert_eq!(count_primary_user_things(c), 1);
+
+        let (c, _) = CertBuilder::new()
+            .add_user_attribute(ua_foo.clone())
+            .add_user_attribute(ua_bar.clone())
+            .generate()?;
+        assert_eq!(count_primary_user_things(c), 1);
+
+        let (c, _) = CertBuilder::new()
+            .add_userid("foo")
+            .add_user_attribute(ua_foo.clone())
+            .generate()?;
+        let vc = c.with_policy(p, None)?;
+        assert_eq!(vc.primary_userid()?.binding_signature().primary_userid(),
+                   Some(true));
+        assert_eq!(vc.primary_user_attribute()?.binding_signature().primary_userid(),
+                   None);
+        assert_eq!(count_primary_user_things(c), 1);
+
+        let (c, _) = CertBuilder::new()
+            .add_user_attribute(ua_foo.clone())
+            .add_userid("foo")
+            .generate()?;
+        let vc = c.with_policy(p, None)?;
+        assert_eq!(vc.primary_userid()?.binding_signature().primary_userid(),
+                   Some(true));
+        assert_eq!(vc.primary_user_attribute()?.binding_signature().primary_userid(),
+                   None);
+        assert_eq!(count_primary_user_things(c), 1);
+
+        let (c, _) = CertBuilder::new()
+            .add_userid("foo")
+            .add_userid_with(
+                "buz",
+                SignatureBuilder::new(positive).set_primary_userid(false)?)?
+            .add_userid_with(
+                "bar",
+                SignatureBuilder::new(positive).set_primary_userid(true)?)?
+            .add_userid_with(
+                "baz",
+                SignatureBuilder::new(positive).set_primary_userid(true)?)?
+            .generate()?;
+        let vc = c.with_policy(p, None)?;
+        assert_eq!(vc.primary_userid()?.value(), b"bar");
+        assert_eq!(count_primary_user_things(c), 1);
+
+        Ok(())
+    }
 }
