@@ -245,10 +245,8 @@ impl S2K {
     /// Returns whether this S2K mechanism is supported.
     pub fn is_supported(&self) -> bool {
         use self::S2K::*;
-        #[allow(deprecated)]
-        match self {
-            Simple { .. } | Salted { .. } | Iterated { .. } => true,
-            _ => false,
+        #[allow(deprecated)] {
+            matches!(self, Simple { .. } | Salted { .. } | Iterated { .. })
         }
     }
 
@@ -422,7 +420,7 @@ mod tests {
             cipher_algo: SymmetricAlgorithm,
             password: Password,
             key_hex: &'a str,
-        };
+        }
 
         // Note: this test only works with SK-ESK packets that don't
         // contain an encrypted session key, i.e., the session key is
