@@ -29,7 +29,7 @@
 //!
 //! Describes a key that has been designated to issue revocation signatures.
 //!
-//! # `ServerPreferences`
+//! # `KeyServerPreferences`
 //!
 //! Describes preferences regarding to key servers.
 //!
@@ -145,11 +145,13 @@ impl PublicKeyAlgorithm {
     /// ```
     pub fn for_signing(&self) -> bool {
         use self::PublicKeyAlgorithm::*;
-        #[allow(deprecated)]
-        match &self {
-            RSAEncryptSign | RSASign | DSA | ECDSA | ElGamalEncryptSign
-                | EdDSA => true,
-            _ => false,
+        #[allow(deprecated)] {
+            matches!(self, RSAEncryptSign
+                     | RSASign
+                     | DSA
+                     | ECDSA
+                     | ElGamalEncryptSign
+                     | EdDSA)
         }
     }
 
@@ -167,11 +169,12 @@ impl PublicKeyAlgorithm {
     /// ```
     pub fn for_encryption(&self) -> bool {
         use self::PublicKeyAlgorithm::*;
-        #[allow(deprecated)]
-        match &self {
-            RSAEncryptSign | RSAEncrypt | ElGamalEncrypt | ECDH
-                | ElGamalEncryptSign => true,
-            _ => false,
+        #[allow(deprecated)] {
+            matches!(self, RSAEncryptSign
+                     | RSAEncrypt
+                     | ElGamalEncrypt
+                     | ECDH
+                     | ElGamalEncryptSign)
         }
     }
 
@@ -1139,6 +1142,15 @@ pub enum SignatureType {
     /// Positive certification of a User ID and Public-Key packet.
     PositiveCertification,
 
+    /// Attestation Key Signature (proposed).
+    ///
+    /// Allows the certificate owner to attest to third party
+    /// certifications. See [Section 5.2.3.30 of RFC 4880bis] for
+    /// details.
+    ///
+    ///   [Section 5.2.3.30 of RFC 4880bis]: https://tools.ietf.org/html/draft-ietf-openpgp-rfc4880bis-10.html#section-5.2.3.30
+    AttestationKey,
+
     /// Subkey Binding Signature
     SubkeyBinding,
     /// Primary Key Binding Signature
@@ -1163,11 +1175,6 @@ pub enum SignatureType {
 }
 assert_send_and_sync!(SignatureType);
 
-/// An attested key signature.
-#[allow(non_upper_case_globals)]
-pub(crate) const SignatureType__AttestedKey: SignatureType =
-    SignatureType::Unknown(0x16);
-
 
 impl From<u8> for SignatureType {
     fn from(u: u8) -> Self {
@@ -1179,6 +1186,7 @@ impl From<u8> for SignatureType {
             0x11 => SignatureType::PersonaCertification,
             0x12 => SignatureType::CasualCertification,
             0x13 => SignatureType::PositiveCertification,
+            0x16 => SignatureType::AttestationKey,
             0x18 => SignatureType::SubkeyBinding,
             0x19 => SignatureType::PrimaryKeyBinding,
             0x1f => SignatureType::DirectKey,
@@ -1202,6 +1210,7 @@ impl From<SignatureType> for u8 {
             SignatureType::PersonaCertification => 0x11,
             SignatureType::CasualCertification => 0x12,
             SignatureType::PositiveCertification => 0x13,
+            SignatureType::AttestationKey => 0x16,
             SignatureType::SubkeyBinding => 0x18,
             SignatureType::PrimaryKeyBinding => 0x19,
             SignatureType::DirectKey => 0x1f,
@@ -1232,6 +1241,8 @@ impl fmt::Display for SignatureType {
                 f.write_str("CasualCertification"),
             SignatureType::PositiveCertification =>
                 f.write_str("PositiveCertification"),
+            SignatureType::AttestationKey =>
+                f.write_str("AttestationKey"),
             SignatureType::SubkeyBinding =>
                 f.write_str("SubkeyBinding"),
             SignatureType::PrimaryKeyBinding =>
